"use strict";
exports.__esModule = true;
var fs_1 = require("fs");
var prettier_1 = require("prettier");
function write(filename, str) {
    var s = null;
    while (s !== str) {
        s = str;
        str = prettier_1.format(str, {
            bracketSpacing: false,
            singleQuote: true,
            trailingComma: 'all',
            parser: 'typescript',
            filepath: filename
        });
    }
    var old = null;
    try {
        old = fs_1.readFileSync(filename, 'utf8');
    }
    catch (ex) {
        if (ex.code !== 'ENOENT') {
            throw ex;
        }
    }
    if (old !== str) {
        fs_1.writeFileSync(filename, str);
    }
}
var builders = fs_1.readdirSync(__dirname + '/walkers')
    .filter(function (filename) { return filename !== 'index.ts'; })
    .map(function (filename) { return filename.replace(/\.ts$/, ''); });
write(__dirname + '/walkers/index.ts', "\nimport {writeFileSync} from 'fs';\nimport * as bt from 'babel-types';\nimport Context from '../Context';\n" + builders.map(function (n) { return "import walk" + n + " from './" + n + "';"; }).join('\n') + "\n\nexport default function walk(node: bt.Statement, ctx: Context): bt.Identifier[] {\n  switch (node.type) {\n" + builders
    .map(function (n) { return "    case '" + n + "': return walk" + n + "(node as bt." + n + ", ctx);"; })
    .join('\n') + "\n    default:\n      writeFileSync(\n        __dirname + '/' + node.type + '.ts',\n        \"import * as bt from 'babel-types';\\n\" +\n        \"import Context from '../Context';\\n\\n\" +\n        \"export default function walk\" + node.type + \"(node: bt.\" + node.type + \", ctx: Context): bt.Identifier[] {\\n\" +\n        \"  throw ctx.getError('Unsupported type \" + node.type + \"', node);\\n\" +\n        \"}\\n\",\n      );\n      throw ctx.getError('Unsupported node type ' + node.type, node);\n  }\n}\n");
var printers = fs_1.readdirSync(__dirname + '/printers')
    .filter(function (filename) { return filename !== 'index.ts'; })
    .map(function (filename) { return filename.replace(/\.ts$/, ''); });
function special(n) {
    return n === 'SingleImportDeclaration' || n === 'SingleVariableDeclaration';
}
write(__dirname + '/printers/index.ts', "\nimport {writeFileSync} from 'fs';\nimport * as bt from 'babel-types';\nimport Context, {SingleImportDeclaration, SingleVariableDeclaration} from '../Context';\n" + printers.map(function (n) { return "import print" + n + " from './" + n + "';"; }).join('\n') + "\n\nexport default function print(node: bt.Node, ctx: Context): string {\n  switch (node.type) {\n" + printers
    .map(function (n) {
    return "    case '" + n + "': return print" + n + "(node as " + (special(n) ? '' : 'bt.') + n + ", ctx);";
})
    .join('\n') + "\n    default:\n      writeFileSync(\n        __dirname + '/' + node.type + '.ts',\n        \"import * as bt from 'babel-types';\\n\" +\n        \"import Context from '../Context';\\n\" +\n        \"import print from './';\\n\\n\" +\n        \"export default function print\" + node.type + \"(node: bt.\" + node.type + \", ctx: Context): string {\\n\" +\n        \"  throw ctx.getError('Unsupported type \" + node.type + \"', node);\\n\" +\n        \"}\\n\",\n      );\n      throw ctx.getError('Unsupported node type ' + node.type, <bt.Node>node);\n  }\n}\n");
