"use strict";
exports.__esModule = true;
var fs_1 = require("fs");
var babylon_1 = require("babylon");
var lsr_1 = require("lsr");
var micromatch_1 = require("micromatch");
var prettier_1 = require("prettier");
var Context_1 = require("./Context");
var walkers_1 = require("./walkers");
function convertFile(filename, options) {
    if (options === void 0) { options = {}; }
    var src = fs_1.readFileSync(filename, 'utf8');
    var ast = babylon_1.parse(src, {
        sourceType: 'module',
        sourceFilename: filename,
        plugins: [
            'jsx',
            'flow',
            'classConstructorCall',
            'doExpressions',
            'objectRestSpread',
            'classProperties',
            'exportExtensions',
            'asyncGenerators',
            'functionBind',
            'dynamicImport',
        ]
    });
    var ctx = new Context_1["default"](filename, src);
    ast.program.body.map(function (statement) { return walkers_1["default"](statement, ctx); });
    return prettier_1.format(ctx.output.toString(options), {
        singleQuote: true,
        trailingComma: 'all',
        parser: 'typescript',
        filepath: filename.replace(/.js(.flow)?$/, '.d.ts')
    });
}
exports.convertFile = convertFile;
function convertDirectory(dirname, patterns, ignorePatterns, options) {
    if (ignorePatterns === void 0) { ignorePatterns = []; }
    if (options === void 0) { options = {}; }
    lsr_1.lsrSync(process.cwd()).forEach(function (entry) {
        if (entry.isFile() &&
            patterns.some(function (p) { return micromatch_1.isMatch(entry.path.substr(2), p); }) &&
            !ignorePatterns.some(function (p) { return micromatch_1.isMatch(entry.path.substr(2), p); })) {
            var result = convertFile(entry.fullPath, options);
            fs_1.writeFileSync(entry.fullPath.replace(/.js(.flow)?$/, '.d.ts'), result);
        }
    });
}
exports.convertDirectory = convertDirectory;
