"use strict";
exports.__esModule = true;
var bt = require("babel-types");
var _1 = require("./");
// LVal = Identifier | MemberExpression | RestElement | AssignmentPattern | ArrayPattern | ObjectPattern;
function isOptional(node) {
    if (bt.isAssignmentPattern(node)) {
        return true;
    }
    return !!node.optional;
}
function printTypeAnnotation(node, ctx) {
    if (!node.typeAnnotation) {
        throw ctx.getError('This function is exported, so it needs a type annotation', node);
    }
    return _1["default"](node.typeAnnotation, ctx);
}
function printProperty(p, ctx) {
    if (p.type === 'ObjectProperty') {
        var a = p;
        if (!bt.isIdentifier(a.key)) {
            throw ctx.getError('Expected properties of ObjectExpression to be identifiers or rest elmenets', p);
        }
        return a.key.name;
    }
    if (!bt.isIdentifier(p.argument)) {
        throw ctx.getError('Expected rest element to be an identifier', p);
    }
    return "..." + p.argument.name;
}
function printParam(p, isOptional, ctx) {
    var optional = isOptional ? '?' : '';
    if (bt.isIdentifier(p)) {
        return p.name + optional + ': ' + printTypeAnnotation(p, ctx);
    }
    if (bt.isAssignmentPattern(p)) {
        return printParam(p.left, isOptional, ctx);
    }
    if (bt.isRestElement(p)) {
        if (!bt.isIdentifier(p.argument)) {
            throw ctx.getError('Expected rest param to be an identifier', p);
        }
        return "..." + p.argument.name + ": " + printTypeAnnotation(p, ctx);
    }
    if (bt.isObjectPattern(p)) {
        return "{" + p.properties
            .map(function (p) { return printProperty(p, ctx); })
            .join(', ') + "}" + optional + ": " + printTypeAnnotation(p, ctx);
    }
    throw ctx.getError('Unsupported arg type ' + p.type, p);
}
function printParams(params, ctx) {
    return params
        .map(function (p, i) {
        return printParam(p, params.slice(i).every(isOptional), ctx);
    })
        .join(',');
}
exports.printParams = printParams;
function printFunctionDeclaration(node, ctx) {
    if (!node.id) {
        throw ctx.getError('This function does not have an id.', node);
    }
    var name = node.id.name;
    var typeParameters = node.typeParameters
        ? _1["default"](node.typeParameters, ctx)
        : '';
    var params = printParams(node.params, ctx);
    if (!node.returnType) {
        throw ctx.getError('This function is exported, so it needs a return type.', node);
    }
    var returnType = _1["default"](node.returnType, ctx);
    return "declare function " + name + typeParameters + "(" + params + "): " + returnType + ";";
}
exports["default"] = printFunctionDeclaration;
