"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
exports.__esModule = true;
var bt = require("babel-types");
var _1 = require("./");
function printSingleVariableDeclaration(node, ctx) {
    var id = node.declaration.id;
    if (!bt.isIdentifier(id)) {
        throw ctx.getError('Expected id of variable to be an Identifier', node.declaration);
    }
    if (id.typeAnnotation) {
        return "declare " + node.kind + " " + id.name + ": " + _1["default"](id.typeAnnotation, ctx) + ";";
    }
    if (!node.declaration.init) {
        throw ctx.getError('This variable gets exported, so it must have a type annotation or an init.', node.declaration);
    }
    if (bt.isCallExpression(node.declaration.init) &&
        bt.isIdentifier(node.declaration.init.callee, { name: 'curry' })) {
        var args = node.declaration.init.arguments;
        var arg = args[0];
        if (!args.length || !bt.isIdentifier(arg)) {
            throw ctx.getError('The special curry function is only supported with a single argument.', node.declaration.init);
        }
        var declarations = ctx.declarations.get(arg.name);
        if (!declarations || !declarations.some(function (d) { return bt.isFunctionDeclaration(d); })) {
            throw ctx.getError('Could not find a function declaration named ' + arg.name + '.', arg);
        }
        return declarations
            .map(function (d) {
            if (!bt.isFunctionDeclaration(d)) {
                return '';
            }
            var name = id.name;
            var params = d.params;
            var updatedFunction = __assign({}, d, { id: __assign({}, id) });
            var output = [];
            for (var numberPassedAlready = params.length - 1; numberPassedAlready >= 0; numberPassedAlready--) {
                for (var extraArgsPassed = 1; extraArgsPassed <= params.length - numberPassedAlready; extraArgsPassed++) {
                    updatedFunction.id.name =
                        name +
                            (numberPassedAlready === 0 ? '' : 'With' + numberPassedAlready);
                    ctx.output.printed.add(updatedFunction.id.name);
                    updatedFunction.params = params.slice(numberPassedAlready, numberPassedAlready + extraArgsPassed);
                    updatedFunction.returnType =
                        numberPassedAlready + extraArgsPassed < params.length
                            ? bt.typeAnnotation(bt.typeofTypeAnnotation(__assign({}, id, { name: id.name +
                                    'With' +
                                    (numberPassedAlready + extraArgsPassed) })))
                            : d.returnType;
                    output.push(_1["default"](updatedFunction, ctx));
                }
            }
            return output.join('\n');
        })
            .join('\n');
    }
    var type = _1["default"](node.declaration.init, ctx);
    return "declare " + node.kind + " " + id.name + ": " + type + ";";
}
exports["default"] = printSingleVariableDeclaration;
